@@ -37,6 +37,8 @@
 #include "galago.h"
 #include "shared.h"
 
+static GList *ctxts = NULL;
+
 static void
 context_free (FeedContext *context)
 {
@@ -64,6 +66,8 @@ connection_dead (gpointer data, GObject *object)
 
   d(g_printerr (G_STRLOC ": connection dead\n"));
   
+  ctxts = g_list_remove (ctxts, context);
+
   /* We got here as the context died, so nullify it */
   context->conn = NULL;
   context_free (context);
@@ -103,6 +107,9 @@ new_context (const char *service, const char *object_path)
   if (context->conn) {
     /* Get called when the connection object disconnects from the bus */
     g_object_weak_ref (G_OBJECT (context->conn), connection_dead, context);
+
+    ctxts = g_list_prepend (ctxts, context);
+
     /* This is a hash from integers to GalagoAccount objects */
     return context;
   } else {
@@ -111,24 +118,15 @@ new_context (const char *service, const char *object_path)
   }
 }
 
-/**
- * DBus message filter, listening for Telepathy NewConnection signals.
- */
 static DBusHandlerResult
-signal_filter (DBusConnection *connection,
-               DBusMessage    *message,
-               void           *user_data)
+parse_new_connection (DBusConnection *connection,
+		      DBusMessage    *message,
+		      void           *user_data)
 {
   DBusError error;
   char *bus_name, *object_path;
   FeedContext *context;
   
-  if (!dbus_message_is_signal (message,
-                               TP_IFACE_CONN_MGR_INTERFACE,
-                               "NewConnection")) {
-    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-  }
-  
   dbus_error_init (&error);
   
   if (!dbus_message_get_args (message, &error,
@@ -139,7 +137,7 @@ signal_filter (DBusConnection *connection,
     dbus_error_free (&error);
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
   }
-
+  
   context = new_context (bus_name, object_path);
   if (context) {
     send_presence_for_connection (context);
@@ -148,6 +146,68 @@ signal_filter (DBusConnection *connection,
   return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+parse_members_changed (DBusConnection *connection,
+		       DBusMessage    *message,
+		       void           *user_data)
+{
+  DBusError error;
+  char *mess;
+  int *added, *removed, *local_pending, *remote_pending;
+  int n_added, n_removed, n_local, n_remote;
+  int actor, reason;
+  GList *c;
+
+  dbus_error_init (&error);
+
+  if (!dbus_message_get_args (message, &error,
+			      DBUS_TYPE_STRING, &mess,
+			      DBUS_TYPE_ARRAY, DBUS_TYPE_UINT32, 
+			      &added, &n_added,
+			      DBUS_TYPE_ARRAY, DBUS_TYPE_UINT32, 
+			      &removed, &n_removed,
+			      DBUS_TYPE_ARRAY, DBUS_TYPE_UINT32, 
+			      &local_pending, &n_local,
+			      DBUS_TYPE_ARRAY, DBUS_TYPE_UINT32, 
+			      &remote_pending, &n_remote,
+			      DBUS_TYPE_UINT32, &actor,
+			      DBUS_TYPE_UINT32, &reason,
+			      DBUS_TYPE_INVALID)) {
+    g_warning ("Cannot parse MembersChanged message: %s", error.message);
+    dbus_error_free (&error);
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+
+  /* FIXME: Work out which context this message relates to */
+  for (c = ctxts; c; c = c->next) {
+    send_presence_for_connection (c->data);
+  }
+  return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+/**
+ * DBus message filter, listening for Telepathy NewConnection signals.
+ */
+static DBusHandlerResult
+signal_filter (DBusConnection *connection,
+               DBusMessage    *message,
+               void           *user_data)
+{
+  if (dbus_message_is_signal (message,
+			      TP_IFACE_CONN_MGR_INTERFACE,
+			      "NewConnection")) {
+    return parse_new_connection (connection, message, user_data);
+  }
+  
+  if (dbus_message_is_signal (message,
+			      TP_IFACE_CHANNEL_INTERFACE_GROUP,
+			      "MembersChanged")) {
+    return parse_members_changed (connection, message, user_data);
+  }
+  
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}  
+
 /**
  * Listen for new Telepathy connections being created, so their presence can be
  * injected into Galago.
@@ -243,12 +303,12 @@ connect_to_existing (void)
     g_strdelimit (path, ".", '/');
 
     d(g_printerr (G_STRLOC": connecting to %s\n", service));
-
+    
     context = new_context (service, path);
     if (context) {
       /* We need to create the hash now as it's lazy created normally */
       context->account_hash = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
-     send_presence_for_connection (context);
+      send_presence_for_connection (context);
     }
 
     g_free (service);
